#!/bin/sh

source /etc/rc.d/rc.common
navico_env_init

DEFAULT_APPARGS=""
APPENV=""
EVDEVINPUT_PARAMS="-plugin evdevhotplug"\
":dir=/dev/input"\
":mask=\"remotecontrol.*|mouse.*\""\
":internalkey=/dev/input/by-keymap/internal"\
":externalkey=/dev/input/by-keymap/external"
case $FAMILYID in
  Kiwi)
    TARGETUSERNAME=nos
    APPNAME=MercuryBlackBoxApp
    DEFAULT_APPARGS="-platform linuxfb $EVDEVINPUT_PARAMS"
    ;;
  Hobart)
    TARGETUSERNAME=nos
    APPNAME=InstrumentApp
    DEFAULT_APPARGS="-platform linuxfb $EVDEVINPUT_PARAMS"
    ;;
  Repower)
    TARGETUSERNAME=nos
    APPNAME=InstrumentApp
    DEFAULT_APPARGS="-platform linuxfb $EVDEVINPUT_PARAMS"
    case "$SUBFAMILYID" in
    1|2) # Viking, Chunky
      DEFAULT_APPARGS="$DEFAULT_APPARGS -plugin navicorotary:/dev/input/rotary"
      ;;
    esac
    ;;
  Sylvester)
    TARGETUSERNAME=nos
    APPNAME=VioletApp
    DEFAULT_APPARGS=""
    ;;
  *)
    TARGETUSERNAME=nos
    APPNAME=NOSApp
    DEFAULT_APPARGS="-platform linuxfb"
    ;;
esac

if [ "${DEFAULT_APPARGS/platform linuxfb}" != "$DEFAULT_APPARGS" ]; then
    APP_ENV="$APP_ENV QT_QPA_FB_DISABLE_INPUT=1"
fi

HOMEDIR=/home/${TARGETUSERNAME}
BOOTMODE_FILE=${HOMEDIR}/boot_mode
LAUNCHER=/usr/bin/navico-applauncher
APP=$(readlink -f /usr/bin/$APPNAME)
APPLOGFILE=${HOMEDIR}/NOS/.log/$APPNAME
APPARGS=${DEFAULT_APPARGS}
TESTAPP=/usr/sbin/TestApp
CLIENTCHECKER=/usr/sbin/ClientChecker
TIMEOUT=40
CONFIG_PARTITION=/etc/NOS
NOS_RESOURCE=/usr/share/NOS/resources
TMP_DIR=/tmp/NOS
RUN_DIR=/var/run
BOOTKEYS=/dev/shm/bootkeys
CAPS="cap_net_admin,cap_sys_nice,cap_sys_time,cap_sys_tty_config+ep"

## The purpose of this section of initialization code is find the
## paths.ini file and read platform values for various settings, using
## reasonable defaults if it fails for any reason.
# TODO: turn this into a couple of calls to a generic function, parsepathsini()
SETTINGSINIPATH="${HOMEDIR}"
RESOURCESPATH=/usr/share/NOS/resources

if [ -r ${BOOTKEYS} ]; then
  PASSBOOTKEYS="--BOOTKEYS=$(cat ${BOOTKEYS})"
fi

if [ -r "${HOMEDIR}/paths.ini" ]; then
  # SettingsIni=./
  if grep -qi ^SettingsIni= "${HOMEDIR}/paths.ini"; then
    SETTINGSINIPATH=$(grep -i ^SettingsIni= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${SETTINGSINIPATH}" ]; then
      SETTINGSINIPATH="$(cd "${HOMEDIR}" && cd "${SETTINGSINIPATH}" && pwd)"
    fi
  fi
  # Resources=/usr/share/NOS/resources
  if grep -qi ^Resources= "${HOMEDIR}/paths.ini"; then
    RESOURCESPATH=$(grep -i ^Resources= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${RESOURCESPATH}" ]; then
      RESOURCESPATH="$(cd "${HOMEDIR}" && cd "${RESOURCESPATH}" && pwd)"
    fi
  fi
  # ChartData=/usr/share/NOS/charts/
  #if grep -qi ^ChartData= "${HOMEDIR}/paths.ini"; then
  #  CHARTDATAPATH=$(grep -i ^ChartData= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
  #  if [ -n "${CHARTDATAPATH}" ]; then
  #    CHARTDATAPATH="$(readlink ${CHARTDATAPATH%/})"
  #    MAP_PARTITION="${CHARTDATAPATH%/charts*}"
  #  fi
  #fi
fi

if [ -e /etc/navico-coredump.conf ]; then
    . /etc/navico-coredump.conf
fi

deleteflags() {
  rm $TMP_DIR/app-restart
  rm $TMP_DIR/poweroff
  rm $TMP_DIR/app-stop
}

movelogs() {
  if [ -e ${APPLOGFILE} ]; then
    [ -e ${APPLOGFILE}.2 ] && rm ${APPLOGFILE}.2
    [ -e ${APPLOGFILE}.1 ] && mv ${APPLOGFILE}.1 ${APPLOGFILE}.2
    mv ${APPLOGFILE} ${APPLOGFILE}.1
  fi
}

addlogentry() {
  echo $@
  if [ -e /dev/shm/enablecrashlog ]; then
    echo "rc.app: $@" >> ${APPLOGFILE}
    echo "$@" | syslogger -q -n rc.app
  fi
}

enabledebuglogin() {
  # Launch telnetd. It won't launch a second time, because it's bind to the port fails
  /usr/sbin/telnetd

  # Launch ssh
  [ -x /etc/rc.d/rc.sshd ] && /etc/rc.d/rc.sshd start
}

init_homedir() {
  mkdir -p $HOMEDIR.new
  cp -a /etc/skel/* $HOMEDIR.new/

  # By default, the root of new fs is owned by root. Change home dir to be owned by correct user
  chown -R ${TARGETUSERNAME}:users $HOMEDIR.new
  sync

  mv $HOMEDIR.new $HOMEDIR
  sync
}

restore_defaults() {
  # "Restore NOS defaults" - create a fresh home dir for $TARGETUSERNAME only
  # A full factory reset recreates the /home partition (refer rc.S)
  mv $HOMEDIR $HOMEDIR.old
  init_homedir
  rm -fr $HOMEDIR.old
  sync
}

read_boot_mode() {
  # Boot mode:
  #   1	TestApp
  #   2	NOSApp / normal boot (default)
  #   3	Client checker / burn-in
  BOOTMODE=$(grep -s '^[1-3]$' ${BOOTMODE_FILE} || echo 2)
}

# Create symlinks for compatibility with other products that have a dedicated
# partition for charts and splash screen images
detect_custom_splash() {
  if [ -d /etc/NOS/splash-images ]; then
    if [ ! -h /media/factorydata/splash-images ]; then
      rm -rf /media/factorydata/splash-images
      ln -s /etc/NOS/splash-images/ /media/factorydata/
    fi
  fi
}

launch_nosapp() {
  # Make sure the mcp251x module is loaded (used for third can controller).
  # NOTE: This may be un-loaded by the testapp so it can talk to the lcd spi
  case $FAMILYID in
    Kiwi) modprobe mcp251x ;;
  esac

  # Mediadeamon is stupid - its not really a deamon, but actually connects to NOSApp which is
  # acting as the server. So it needs to restart each time nosapp does
  if [ -x /etc/rc.d/rc.mediadaemon ]; then
    /etc/rc.d/rc.mediadaemon restart
  fi

  # Rotate logs on every start
  movelogs

  detect_custom_splash

  unset EXITCODE
  unset LAUNCHER_PID

  case $FAMILYID in
    Sylvester)
        #disable IRQ from bank0
        memtool 80000918=4
        #disable updating bank0 irq status reg
        memtool 80019008=1fffffff
        #reset bank0 irq status reg
        memtool 80019408=1fffffff
        # Not using applauncher for VioletApp.
        ${APP} "${PASSBOOTKEYS}" ${APPARGS}
        EXITCODE=$?
        unset PASSBOOTKEYS
      ;;
    *)
      local app_env
      if [ -n "${APP_ENV:-}" ]; then
          app_env="$APP_ENV "
      fi
      # Optionally including keymap parameters if the keymap file exists.
      local app_args=$APPARGS
      local resources_path=/usr/share/NOS/resources
      local keymap_re

      case "$CODENAME" in
      chunkyrec)
          keymap_re="${CODENAME}.qmap"
          ;;
      viking*|chunky*)
          keymap_re="simrad.qmap"
          ;;
      *)
          # All other devices we should expect a single qmap on target
          keymap_re="*.qmap"
          ;;
      esac

      local keymap_file=$(find "$resources_path" -maxdepth 1 -name "$keymap_re" | head -n1)

      if [ -n "$keymap_file" ]; then
        local extra_args="internalkeymap=${keymap_file}:"
        app_args=${app_args/evdevhotplug/evdevhotplug:$extra_args}
      fi
      if [ -e /dev/shm/enablecrashlog ]; then
          su -c "${app_env}${LAUNCHER} --timeout ${TIMEOUT} \
             --kill \
             --coredumpflag ${NAVICO_COREDUMP_FLAG} \
             --socket ${RUN_DIR}/app_monitor_daemon \
             -- ${APP} ${app_args} \
             2>&1 | syslogger -n NOSApp" \
             - ${TARGETUSERNAME} &
          LAUNCHER_PID=$!
      else
          su -c "${app_env}${LAUNCHER} --timeout ${TIMEOUT} \
             --kill \
             --coredumpflag ${NAVICO_COREDUMP_FLAG} \
             --socket ${RUN_DIR}/app_monitor_daemon \
             -- ${APP} ${app_args}" \
             - ${TARGETUSERNAME} &
          LAUNCHER_PID=$!
      fi
      sleep 1
      # Watch the nosapp log, and if it gets too big, rotate it
      ( while true; do
          sleep 60
          if [ -e ${APPLOGFILE} ] && [ "$(du ${APPLOGFILE} | awk '{ print $1 }')" -gt 256 ]; then
              movelogs

              # NOSApp responds to SIGUSR2 by closing and reopening its log file
              kill -USR2 $(pidof $APPNAME)
          fi
          done
      ) &
      LOG_WATCH_PID=$!
      ;;
  esac

  if [ "$LAUNCHER_PID" ]; then
    wait $LAUNCHER_PID
  fi

  if [ "$LOG_WATCH_PID" ]; then
    kill $LOG_WATCH_PID
    unset LOG_WATCH_PID
  fi

  read_boot_mode

  # check app stop flag
  if [ "$EXITCODE" = "1" ] || [ -f $TMP_DIR/app-stop ]; then
    addlogentry App Stop Requested
    deleteflags
    break

  # check app restart flag
  elif [ "$EXITCODE" = "2" ] || [ -f $TMP_DIR/app-restart ]; then
    # Only Sylvester requires special handling of a restart request
    if [ "$FAMILYID" = Sylvester ]; then
      # 1 =test app
      echo 1 >${BOOTMODE_FILE}
    fi
    addlogentry App Restart
    unset REBOOTING
    STOPCNT=0
    while pgrep $APPNAME; do
       sleep 1
       STOPCNT=$(($STOPCNT+1))
       if [ "$STOPCNT" -gt 15 ]; then
           if [ ! -e /dev/shm/debug ]; then
               /sbin/reboot
           fi
           REBOOTING=true
       fi
    done
    unset STOPCNT
    deleteflags

    if [ ! -z "$REBOOTING" ]; then
        break
    fi

  # check poweroff flag
  elif [ "$EXITCODE" = "0" ] || [ -f $TMP_DIR/poweroff ]; then
    addlogentry Power Off
    deleteflags
    /sbin/poweroff
    break

  elif [ "2" != "$BOOTMODE" ]; then
    echo Switching to BOOTMODE: $BOOTMODE

  else
    addlogentry App has died. Rebooting

    # Sylvester does not restart properly. Lets just restart the app instead.
    if [ $FAMILYID != Sylvester ]; then
      if [ ! -e /dev/shm/debug ]; then
        # Give log capture a chance to work
        if [ -e /dev/shm/enablecrashlog ]; then
          sleep 5
        fi

        /sbin/reboot
      fi
      break
    fi
  fi
}

init_config_ini() {
    local codename=$CODENAME

    local new_config
    for new_config in "${NOS_RESOURCE}/Config-${codename}-${SUBFAMILYID}.ini" \
                        "${NOS_RESOURCE}/Config-${codename}.ini"; do
        if [ -s "$new_config" ]; then
          local config="${CONFIG_PARTITION}/Config.ini"

          mount -o remount,rw "$CONFIG_PARTITION"
          cat "$new_config" > "$config"
          chmod 644 "$config"
          sync
          mount -o remount,ro "$CONFIG_PARTITION"
        fi
    done
}

start() {
  # Make a default config.ini if one does not exist
  if [ "${FAMILY_ID}" = "Sylvester" ]; then
      # Sylvester just needs a file present.
      if [ ! -s $CONFIG_PARTITION/Config.dat ]; then
        mount -o remount,rw $CONFIG_PARTITION
        touch $CONFIG_PARTITION/Config.dat
        sync
        mount -o remount,ro $CONFIG_PARTITION
      fi
  else
      [ -s $CONFIG_PARTITION/Config.ini ] || init_config_ini
  fi

  if [[ "$(getcap $APP | cut -d " " -f 3)" != "$CAPS" ]] || [[ "$(getcap $LAUNCHER | cut -d " " -f 3)" != "$CAPS" ]]; then
    echo "Setting application capabilities..."
    setcap $CAPS $APP
    setcap $CAPS $LAUNCHER
    sync
  fi

  # Make sure the temp directory actually exists
  if [ ! -d $TMP_DIR ]; then
    mkdir -p $TMP_DIR
  fi
  chown ${TARGETUSERNAME}:users ${TMP_DIR}

  # Make sure the home directory actually exists
  if [ ! -e $HOMEDIR ]; then
    init_homedir
  fi

  cd "$HOMEDIR"

  ( while true; do
    read_boot_mode

    if [ -r ${BOOTKEYS} ]; then
      case "${FAMILYID}" in
        Kiwi)
          if [ "$(cat ${BOOTKEYS} )" = "001" ]; then
            BOOTMODE=1
          fi
          ;;
        Sylvester)
          if [ "$(cat ${BOOTKEYS} )" = "108" ]; then
            BOOTMODE=1
          fi
          ;;
        Hobart|Repower)
          if [ "$(cat ${BOOTKEYS} )" = "50" ]; then
            BOOTMODE=1
          fi
          ;;
      esac

      mv ${BOOTKEYS} ${BOOTKEYS}.old
    fi

    case ${BOOTMODE} in
      1)
        case "${FAMILYID}" in
          Sylvester)
            rm -f ${BOOTMODE_FILE}
            # The "TestApp" for VioletApp is just a special mode of the application.
            mount -o remount,rw ${CONFIG_PARTITION}
            APPARGS=TEST
            launch_nosapp
            mount -o remount,ro ${CONFIG_PARTITION}
            APPARGS=${DEFAULT_APPARGS}
            ;;
          *)
            # TestApp
            # Currently there are some requirements that must be met - refer http://nzwiki.marine.net.int/wiki/StingrayII/TestApp
            enabledebuglogin

            mount -o remount,rw ${CONFIG_PARTITION}

            # Stop mediadaemon, and ensure all the auto-mounted SD cards are un-mounted
            /etc/rc.d/rc.mediadaemon stop
            for MOUNTPOINT in /media/mmcblk*; do
                umount $MOUNTPOINT
            done
            unset MOUNTPOINT

            # Make sure the mcp251x module is loaded (used for third can controller).
            case $FAMILYID in
              Kiwi) modprobe mcp251x ;;
            esac


            # Still run as root, but wrap inside su to set environment correctly.
            # Pass the location of paths.ini to TestApp so it has a chance to be
            # smarter about where to find things
            local app_env
            if [ -n "${APP_ENV:-}" ]; then
                app_env="$APP_ENV "
            fi
            if [ -r ${HOMEDIR}/paths.ini ]; then
              su -c "${app_env}${TESTAPP} $APPARGS ${HOMEDIR}/paths.ini" -
            else
              su -c "${app_env}${TESTAPP} $APPARGS " -
            fi
            EXITCODE=$?

            sync
            mount -o remount,ro ${CONFIG_PARTITION}

            # For TestApp exit codes, refer TestApp/Widgets/tSpecialModes.h
            case ${EXITCODE} in
              0|16)
                echo "Normal mode"
                restore_defaults # and implied: rm -f ${BOOTMODE_FILE}; sync
                ;;
              17)
                echo "Demo mode"
                # <TODO> ??!
                ;;
    #          18) - no 'burn-in' needed on kiwi
    #            echo "Burnin mode"
    #            echo 3 >${BOOTMODE_FILE}
    #            sync
    #            ;;
              18|19)
                echo "TestMode"
                echo 1 >${BOOTMODE_FILE}
                sync
                ;;
              20)
                echo "Reboot"
                /sbin/reboot
                exit
                ;;
               *)
                # Signals are 128+SIGNUM (eg 143 for SIGTERM)
                echo "$0: Unhandled exit code ${EXITCODE}"
                break
                ;;
            esac
          ;;
          esac
        ;;

      2)
        # NOSApp / normal boot
        
        # VLT-626:  Sylvester needs a patch-up to fix a factory problem with 
        # serial numbers not being programmed on a (large) batch of units.
        case "${FAMILYID}" in
          Sylvester)
            if [ ! -s ${CONFIG_PARTITION}/AltID.dat ]; then
              mount -o remount,rw ${CONFIG_PARTITION}
              hexdump -n8 -e\"%04u\" /dev/urandom > ${CONFIG_PARTITION}/AltID.dat
              sync
              mount -o remount,ro ${CONFIG_PARTITION}
            fi
            APPARGS=${DEFAULT_APPARGS}
            ;;
          *)
            ;;
         esac
        
        launch_nosapp
        ;;

#      3)
#        # Client checker / burn-in
#        enabledebuglogin
#
#        mount -o remount,rw ${MAP_PARTITION}
#        mount -o bind ${MAP_PARTITION} /usr/share/NOS
#
#        # ClientChecker needs an IP address to function properly.  Spin.
#        echo "ClientChecker: Waiting for an IP address to appear on eth0."
#        unset IPADDR
#        while [ -z "${IPADDR}" ];
#        do
#          IPADDR=$(
#             ifconfig | \
#             grep -A1 '^eth0[^:]' | \
#             grep "inet addr:" | \
#             sed 's/.*inet addr://; s/ .*//'
#          )
#          sleep 1
#        done
#        unset IPADDR
#
#        # Still run as root, but wrap inside su to set environment correctly.
#        # Set QWS_MOUSE_PROTO to null, so that ClientChecker cannot be killed by pressing on the screen
#        su -c "cd ${HOMEDIR} && export QWS_MOUSE_PROTO="" && ${CLIENTCHECKER} -qws" -
#
#        sync
#        umount /usr/share/NOS
#        mount -o remount,ro ${MAP_PARTITION}
#
#        # Launch NOSApp so the factory can change BOOT_MODE via serial console.  Refer STP-161.
#        launch_nosapp
#        ;;
    esac
    done
  ) &
}

stop() {
  # TODO local kill_list="${APP} ${TESTAPP} ${CLIENTCHECKER}"

  if ! pgrep ${APP} > /dev/null 2>&1; then
    echo "App not running"
    exit 1
  else
    # Force script to exit when NOSApp killed
    # (signals are lost through applauncher)
    touch $TMP_DIR/app-stop

    pkill ${APP}
    while pgrep ${APP} > /dev/null 2>&1; do
      sleep 0.1
    done
  fi
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    restart)
        stop
        start
        ;;
    *)
        echo "Usage: $0 {start | stop | restart}"
        ;;
esac
