#!/bin/sh

set -u

showdialog() {
    local txt=$(cat) # slurp stdin
    local lines=$(printf '%s\n' "$txt" | wc -l)
    local width=$(printf '%s\n' "$txt" | wc -L)
    local attrs=${ANSI_ATTRS:-'97;44'}; # Hi-white on blue background

    width=$((width + 4))
    width=$((width < 16 ? 16 : width))

    local padstr=$(head -c $((width/2)) </dev/zero | tr '\0' ' ')

    printf '%s\n' "$txt" |
    while read; do
        # Center each line
        printf '%s%s\n' "${padstr:$(( ${#REPLY} / 2 ))}" "$REPLY"

    done | sed $'
        1s/^/\33['${attrs}$'m/      # Set initial colour
        s/>/\33[7m /                # Invert bg/fg colours at ">"
        s/</ \33[0;'${attrs}$'m/    # Restore colours at "<"
        s/$/\33[K/                  # Pad to end of line
        ' |
    fbdraw --text -x 50% -y 50% --width $((8*width)) --height $((16*lines))
}

main() {
    local action=${1:-}; shift
    local fbdev=${FRAMEBUFFER:-/dev/fb0}
    local sysfs=/sys/class/graphics/fb${fbdev##*[^0-9]}
    local yres=$(sed -ne 's/.*x\([0-9]\+\).*/\1/p' ${sysfs}/mode)
    local stride=$(cat ${sysfs}/stride)
    local status_msg
    local pidfile='/run/coredump-fbdraw.pid'

    if [ -z "$yres" ]; then
        yres=$(sed -ne 's/.*,\([0-9]\+\).*/\1/p' ${sysfs}/virtual_size)
        yres=$((yres/2))
    fi

    # Stop the animated bar (if running)
    [ -s "$pidfile" ] && kill $(cat "$pidfile") && rm "$pidfile"

    case $action in
        start)
            # Grab an image of the screen at the moment the crash happened.
            #   IndigoApp uses triple buffering, so the display might be panned
            #   Figure this out so we capture the right piece of screen
            local ypan=$(cut -d , -f 2 ${sysfs}/pan)
            dd if=$fbdev of=/tmp/crash-screen-grab.raw bs=$stride skip=$ypan count=$yres 2>/dev/null

            # Return pan to 0,0 so display output is visible
            echo '0,0' > ${sysfs}/pan

            ANSI_ATTRS='97;41' # Hi-white on red background
            status_msg='Creating log, please wait...'
            ;;
        stop)
            # clear the screen
            dd if=/dev/zero of=$fbdev bs=$stride count=$yres 2>/dev/null
            status_msg='Log saved, okay to poweroff.'
            ;;
        *)
            printf 'Usage: %s {start|stop} process-name [addition-debug-text ...]\n' "${0##*/}" >&2
            return 1
            ;;
    esac

    local params=$(printf '\n>APPLICATION CRASHED :(<\n\n%s\n\n%s\n ' "$*" "$status_msg" | showdialog)

    if [ "$action" = "start" ]; then
        # Show an animated bar inside the bottom edge of the dialog
        local yofs=$( echo "$params"|cut -f4) # Window offset
        local xsize=$(echo "$params"|cut -f5) # Window width
        local ysize=$(echo "$params"|cut -f6) # Window height
        # Bar colours (--bar=0xKKLLMM, where KK=bar, LL=border, MM=background.  All 0-15+, ANSI palette)
        fbdraw --bar=$((0x0F0101)) -x 50% -y $((yofs+ysize-14)) --width $((xsize-14)) --height 8 </dev/null >&0 2>&0 &
        echo $! > "$pidfile"
    fi
}

main "$@"
